/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "client.h"
#include "assert/assert.h"
#include "util/memory.h"
#include "console/console.h"
#include "server/server.h"

client_t CLIENT;

void cmdJoin(const consolecmdexec_t *exec) {
  clientconnect_t connect;

  if(exec->argc > 0) {
    connect.type = CLIENT_TYPE_NETWORKED;
    connect.networked.port = SERVER_DEFAULT_PORT;
  } else {
    connect.type = CLIENT_TYPE_SINGLE_PLAYER;
  }

  errorret_t ret = clientConnect(connect);
  if(ret == ERROR_OK) {
    consolePrint("Connected to server");
  } else {
    consolePrint("Failed to connect to server: %s", errorString());
  }
  errorFlush();
}

void cmdLeave(const consolecmdexec_t *exec) {
  clientDisconnect();
}

void clientInit() {
  memoryZero(&CLIENT, sizeof(client_t));

  consoleRegCmd("join", cmdJoin);
  consoleRegCmd("leave", cmdLeave);
}

errorret_t clientConnect(const clientconnect_t connect) {
  errorret_t ret;
  if(CLIENT.state != CLIENT_STATE_DISCONNECTED) {
    return error("Client is already connected");
  }

  CLIENT.type = connect.type;

  switch(connect.type) {
    case CLIENT_TYPE_NETWORKED:
      ret = networkedClientConnect(&CLIENT, connect.networked);
      break;

    default:
      assertUnreachable("Invalid client type");
  }

  if(ret != ERROR_OK) CLIENT.state = CLIENT_STATE_DISCONNECTED;
  return ret;
}

void clientDisconnect() {
  if(CLIENT.state == CLIENT_STATE_DISCONNECTED) return;

  switch(CLIENT.type) {
    case CLIENT_TYPE_NETWORKED:
      networkedClientDisconnect(&CLIENT);
      break;

    default:
      assertUnreachable("Invalid client type");
  }
  
  consolePrint("Client disconnected");
}

void clientProcess() {
  if(CLIENT.state == CLIENT_STATE_DISCONNECTED) return;
}

void clientDispose() {
  clientDisconnect();
}