/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "dusk.h"
#include "packetwelcome.h"
#include "packetdisconnect.h"
#include "packetping.h"

typedef enum {
  PACKET_TYPE_INVALID = 0,
  PACKET_TYPE_WELCOME = 1,
  PACKET_TYPE_DISCONNECT = 2,
  PACKET_TYPE_PING = 3,
} packettype_t;

typedef union {
  packetwelcome_t welcome;
  packetdisconnect_t disconnect;
  packetping_t ping;
} packetdata_t;

typedef struct packet_s {
  packettype_t type;
  uint32_t length;
  packetdata_t data;
} packet_t;

/**
 * Initializes a packet with the given type. This is only to be used by sub
 * initializers, such as packetWelcomeCreate for example.
 * 
 * @param packet Pointer to the packet structure to initialize.
 * @param type The type of the packet.
 * @param length The length of the packet data.
 */
void packetInit(
  packet_t *packet,
  const packettype_t type,
  const uint32_t length
);