/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "packet.h"
#include "util/memory.h"
#include "assert/assert.h"

void packetDisconnectCreate(
  packet_t *packet,
  const packetdisconnectreason_t reason
) {
  packetInit(packet, PACKET_TYPE_DISCONNECT, sizeof(packetdisconnect_t));
  packet->data.disconnect.reason = reason;
}

errorret_t packetDisconnectClient(packet_t *packet) {
  assertNotNull(packet, "Packet is NULL");
  assertTrue(
    packet->type == PACKET_TYPE_DISCONNECT,
    "Packet type is not DISCONNECT"
  );

  if(packet->length != sizeof(packetdisconnect_t)) {
    return error("Disconnect packet length is not correct");
  }

  packetdisconnect_t *data = (packetdisconnect_t *)&packet->data;
  switch(data->reason) {
    case PACKET_DISCONNECT_REASON_UNKNOWN:
      return error("Server disconnected: Unknown reason");
    case PACKET_DISCONNECT_REASON_INVALID_VERSION:
      return error("Server disconnected: Invalid version");
    case PACKET_DISCONNECT_REASON_MALFORMED_PACKET:
      return error("Server disconnected: Malformed packet");
    case PACKET_DISCONNECT_REASON_SERVER_FULL:
      return error("Server disconnected: Server full");
    case PACKET_DISCONNECT_REASON_SERVER_SHUTDOWN:
      return error("Server disconnected: Server shutdown");
    default:
      return error("Server disconnected: Unknown reason");
  }
}