/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "packetqueue.h"
#include "assert/assert.h"
#include "util/memory.h"

void packetQueueInit(packetqueue_t *queue) {
  assertNotNull(queue, "Packet queue is NULL");
  memoryZero(queue, sizeof(packetqueue_t));
}

void packetQueuePushIn(packetqueue_t *queue, const packet_t *packet) {
  assertNotNull(queue, "Packet queue is NULL");
  assertNotNull(packet, "Packet is NULL");
  assertTrue(
    queue->packetsInCount < PACKET_QUEUE_MAX_SIZE, "Packet queue is full"
  );

  queue->packetsIn[queue->packetsInCount++] = *packet;
}

void packetQueuePushOut(packetqueue_t *queue, const packet_t *packet) {
  assertNotNull(queue, "Packet queue is NULL");
  assertNotNull(packet, "Packet is NULL");
  assertTrue(
    queue->packetsOutCount < PACKET_QUEUE_MAX_SIZE, "Packet queue is full"
  );

  queue->packetsOut[queue->packetsOutCount++] = *packet;
}